#coding=utf-8

import datetime

from django.db import models
from django.utils import timezone
from django.conf import settings

from apps.exceptions import CustomError
from apps.account.models import Branch
from apps.customer.models import Customer
from util.wechatpay import WechatPay, WeChatResponse


class Activity(models.Model):
    branch = models.ForeignKey(Branch, verbose_name=u"门店", on_delete=models.PROTECT)
    title = models.CharField(max_length=200, verbose_name=u"标题")
    describe = models.TextField(verbose_name=u'活动介绍')
    end_date = models.DateField(verbose_name=u"截止日期")
    amount = models.FloatField(verbose_name=u"费用", default=0)
    rebate = models.FloatField(verbose_name=u"返利", default=0)
    check_status = models.PositiveSmallIntegerField(choices=settings.CHECK_STATUS_CHOICES, verbose_name=u"审核", editable=False, default=settings.DEFAULT)
    branch_name = models.CharField(max_length=200, verbose_name=u"门店名称", null=True, blank=True)
    branch_tel = models.CharField(max_length=50, verbose_name=u"门店电话", null=True, blank=True)
    branch_address = models.CharField(max_length=200, verbose_name=u"门店地址", null=True, blank=True)
    required_signs = models.CharField(verbose_name=u'必填项', max_length=300, null=True, editable=False)
    create_time = models.DateTimeField(verbose_name=u"添加时间", auto_now_add=True, editable=False)
    enabled = models.BooleanField(verbose_name=u"在用", default=True)
    delete = models.BooleanField(verbose_name=u"删除", default=False)

    @staticmethod
    def getById(id):
        try:
            id = int(id)
        except:
            raise CustomError(u'无效的活动Id')
        instance = Activity.objects.filter(pk=id).first()
        if not instance:
            raise CustomError(u'未找到相应的活动')
        return instance

    class Meta:
        db_table = "activity"
        ordering = ['-id']
        verbose_name = u"活动管理"

    def checkStatus(self):
        if not self.enabled:
            raise CustomError(u'该活动已禁用！')
        if self.delete:
            raise CustomError(u'该活动已删除！')
        if self.check_status != settings.PASS:
            raise CustomError(u'该活动尚未生效！')
        now = datetime.datetime.date(timezone.now())
        if self.end_date and now > self.end_date:
            raise CustomError(u'该活动已过期！')

    @staticmethod
    def getById(id):
        try:
            id = int(id)
        except:
            raise CustomError(u'无效的活动ID')
        instance = Activity.objects.filter(pk=id, delete=False, enabled=True).first()
        if not instance:
            raise CustomError(u'未找到相应的活动')
        return instance


class Pay(models.Model):

    WAIT = 1
    CONFIRM = 2
    CLOSED = 3
    STATUS_CHOICES = (
        (WAIT, u'待付款'),
        (CONFIRM, u'已付款'),
        (CLOSED, u'已关闭'),
    )

    branch = models.ForeignKey(Branch, verbose_name=u"门店", on_delete=models.PROTECT, editable=False)
    no = models.CharField(max_length=64, verbose_name=u"单号")
    create_time = models.DateTimeField(verbose_name=u"创建时间", default=timezone.now)
    customer = models.ForeignKey(Customer, verbose_name=u'客户', on_delete=models.PROTECT)
    status = models.PositiveSmallIntegerField(choices=STATUS_CHOICES, verbose_name=u"支付状态")
    precreate_amount = models.BigIntegerField(verbose_name=u"预支付金额")
    amount = models.BigIntegerField(verbose_name=u"实际支付金额", null=True)

    class Meta:
        db_table = "pay"
        verbose_name = u"支付"
        ordering = ('-id',)
        index_together = (
            'create_time',
            'status',
        )
        unique_together = (
            'no',
        )
        default_permissions = ()

    def payClosed(self):
        if self.status != Pay.WAIT:
            return
        self.status = Pay.CLOSED
        self.save()

    def payConfirm(self, amount):
        if self.status != Pay.WAIT:
            return

        self.status = Pay.CONFIRM
        self.amount = amount
        self.save()
        order = Order.objects.filter(pay=self).first()
        if order:
            order.orderPayConfirm(self.amount)

    @staticmethod
    def getByNo(no):
        instance = Pay.objects.filter(no=no).first()
        if not instance:
            raise CustomError(u'未找到相应的支付单！')
        return instance

    @staticmethod
    def wechatPay(app, branch, customer, amount, openid):
        instance = Pay._addnew(branch, customer, amount)
        return instance, instance._wechatUnifiedOrder(app, openid)

    @staticmethod
    def _addnew(branch, customer, amount):
        if amount <= 0:
            raise CustomError(u'无效的付款金额！')

        no = timezone.now().strftime('%y%m%d%H%M%S') + str(customer.id)
        instance = Pay.objects.create(
            branch=branch,
            no=no,
            customer=customer,
            type=type,
            status=Pay.WAIT,
            precreate_amount=amount
        )
        return instance

    def _wechatUnifiedOrder(self, app, openid):
        wechatpay = WechatPay(app.authorizer_appid, app.agent_num, app.agent_key)
        wechatpay.unifiedOrder(self.no, self.precreate_amount, openid)
        data = wechatpay.getAppString()
        return data


class Order(models.Model):
    DEFAULT = 0
    FINISH = 1
    FAIL = 2
    STATUS_CHOICES = (
        (DEFAULT, u'待付款'),
        (FINISH, u'已完成'),
        (FAIL, u'失败'),
    )
    branch = models.ForeignKey(Branch, verbose_name=u"门店", on_delete=models.PROTECT, editable=False)
    activity = models.ForeignKey(Activity, verbose_name=u"活动", on_delete=models.PROTECT)
    pay = models.ForeignKey(Pay, verbose_name='支付信息', on_delete=models.PROTECT, null=True)
    member = models.ForeignKey(Customer, verbose_name=u"会员", on_delete=models.PROTECT, editable=False)
    amount = models.FloatField(verbose_name=u"费用", default=0)
    status = models.PositiveSmallIntegerField(choices=STATUS_CHOICES, verbose_name=u"状态", default=DEFAULT, editable=False)
    model = models.CharField(max_length=200, verbose_name=u"车型", null=True, blank=True)
    tel = models.CharField(max_length=100, verbose_name=u"电话", null=True, blank=True)
    number = models.CharField(max_length=50, verbose_name=u"车牌号", null=True, blank=True)
    name = models.CharField(max_length=50, verbose_name=u"姓名", null=True, blank=True)
    recommend_member = models.ForeignKey(Customer, verbose_name=u"推荐人", related_name='recommend_member_ref_customer', on_delete=models.PROTECT, null=True, blank=True, editable=False)
    rebate = models.FloatField(verbose_name=u"返利", default=0)
    create_time = models.DateTimeField(verbose_name=u"添加时间", auto_now_add=True, editable=False)
    delete = models.BooleanField(verbose_name=u"删除", default=False)

    class Meta:
        db_table = "order"
        ordering = ['-id']
        verbose_name = u"订单管理"

    def payOrder(self, openid, app):
        # 如果订单上的金额不等于活动金额 (比如下单没有支付，后来活动金额修改，现在活动金额不等于订单上活动金额) 还按下单时保存到额金额支付
        if self.status != Order.DEFAULT:
            raise CustomError(u'订单非代付款状态，禁止付款')
        if self.pay:
            pay_no = self.pay.no
            # 先查询订单状态
            checkRexponse = WeChatResponse(app.authorizer_appid, app.agent_num, app.agent_key)
            total_fee = checkRexponse.orderquery(pay_no)
            if int(total_fee) == int(self.amount):
                wechatpay = WechatPay(app.authorizer_appid, app.agent_num, app.agent_key)
                data = wechatpay.unifiedOrder(openid, pay_no, self.amount)
                return data
            self.pay.payClosed()

        pay, data = Pay.wechatPay(app, self.branch, self.member, self.amount, openid)
        self.pay = pay
        self.save()
        return data

    def orderPayConfirm(self, amount):
        self.status = Order.FINISH
        self.amount = amount
        self.save()
        coupons = ActivityCoupon.objects.filter(activity=self.activity, coupon__enabled=True)
        now = datetime.datetime.date(timezone.now())
        for item in coupons:
            if item.off_type == Coupon.FIXED_DATE:
                end_date = item.end_date
            else:
                end_date = now + datetime.timedelta(days=item.end_days)
            MemberCoupon.objects.create(
                activity=self.activity,
                member=self.member,
                coupon=item,
                receive_date=now,
                end_date=end_date
            )


class Coupon(models.Model):
    FIXED_DATE = 0
    RECEIVE_TIMING = 1
    OFF_TYPE_CHOICES = (
        (FIXED_DATE, u'固定日期'),
        (RECEIVE_TIMING, u'领取计时'),
    )
    branch = models.ForeignKey(Branch, verbose_name=u"门店", on_delete=models.PROTECT)
    name = models.CharField(max_length=200, verbose_name=u"名称")
    off_type = models.PositiveSmallIntegerField(choices=OFF_TYPE_CHOICES, verbose_name=u"核销方式", default=FIXED_DATE)
    end_date = models.DateField(verbose_name=u"截止日期", null=True, blank=True)
    end_days = models.IntegerField(verbose_name=u"截止天数", null=True, blank=True)
    create_time = models.DateTimeField(verbose_name=u"添加时间", auto_now_add=True, editable=False)
    enabled = models.BooleanField(verbose_name=u"在用", default=True)

    class Meta:
        db_table = "coupon"
        ordering = ['-id']
        verbose_name = u"优惠券"

    @staticmethod
    def getById(id):
        try:
            id = int(id)
        except:
            raise CustomError(u'无效的优惠券')
        instance = Coupon.objects.filter(pk=id).first()
        if not instance:
            raise CustomError(u'未找到相应的优惠券')
        return instance


class MemberCoupon(models.Model):
    activity = models.ForeignKey(Activity, verbose_name=u"活动", on_delete=models.PROTECT)
    member = models.ForeignKey(Customer, verbose_name=u"会员", on_delete=models.PROTECT)
    coupon = models.ForeignKey(Coupon, verbose_name=u"优惠券", on_delete=models.PROTECT)
    receive_date = models.DateField(verbose_name=u"领取日期", null=True, blank=True)
    end_date = models.DateField(verbose_name=u"有效期至", null=True, blank=True)
    write_off = models.BooleanField(verbose_name=u"核销", default=False)
    write_off_time = models.DateTimeField(verbose_name=u"核销时间", editable=False, null=True, blank=True)

    class Meta:
        db_table = "member_coupon"
        ordering = ['-id']
        verbose_name = u"会员优惠券"

    def writeOff(self):
        now = datetime.datetime.date(timezone.now())
        if self.write_off:
            raise CustomError(u'该优惠券已核销，禁止重复使用！')
        if self.end_date and now > self.end_date:
            raise CustomError(u'该优惠券已过有效期，禁止使用！')

        self.write_off = True
        self.write_off_time = timezone.now()
        self.save()

    def cancelWriteOff(self):
        if not self.write_off:
            raise CustomError(u'该优惠券尚未核销，禁止撤销！')

        self.write_off = False
        self.write_off_time = None
        self.save()

    @staticmethod
    def getById(id):
        try:
            id = int(id)
        except:
            raise CustomError(u'无效的会员优惠券ID')

        instance = MemberCoupon.objects.filter(pk=id).first()
        if not instance:
            raise CustomError(u'未找到相应的会员优惠券')
        return instance

class ActivityCoupon(models.Model):
    activity = models.ForeignKey(Activity, verbose_name=u"活动", on_delete=models.PROTECT)
    coupon = models.ForeignKey(Coupon, verbose_name=u"优惠券", on_delete=models.PROTECT)

    class Meta:
        db_table = "activity_coupon"
        ordering = ['-id']
        verbose_name = u"活动优惠券"

